function fastBFR(src_col_name,source_name,sigS,sigR,NbSlices,NbIterations,Sigma_Diff)

source = double(imread(source_name))/255;
sig_s  = str2double(sigS);
sig_r  = str2double(sigR)/255;
NbSlic = str2double(NbSlices);
NbIter = str2double(NbIterations);
SDiff  = str2double(Sigma_Diff);

[output_reg,output_std,K] = FBFR_FFT(source,sig_s,sig_r,NbSlic);

%%% Kass & Solomon diffusion step %%%
%

output_std_diffusion = MySelectiveDiffusion(source,output_std,NbIter,SDiff);

%
%%% Kass & Solomon diffusion step %%%

%%% Durand-Dorsey correction for staircasing %%%
%

Kmax = max(K(:));
lambda = 1/8;
sigma_filt = .8;

alpha = max(0, min(1, 1 - (log(K) - log(Kmax))/log(lambda) ));

kernel = exp(-(-2:2).^2 / (2*sigma_filt^2) );
kernel = kernel/sum(kernel(:));

output_std_filt = conv2(mirror(output_std     ,[0 2]),kernel ,'valid');
output_std_filt = conv2(mirror(output_std_filt,[2 0]),kernel','valid');

output_reg_filt = conv2(mirror(output_reg     ,[0 2]),kernel ,'valid');
output_reg_filt = conv2(mirror(output_reg_filt,[2 0]),kernel','valid');

output_std_corr = alpha.*output_std + (1 - alpha).*output_std_filt;
output_reg_corr = alpha.*output_reg + (1 - alpha).*output_reg_filt;

% replace...
output_std = output_std_corr;
output_reg = output_reg_corr;

%
%%% Durand-Dorsey correction for staircasing %%%


imwrite(uint8(255*output_std), 'bilateralstd_base.png');
imwrite(uint8(255*output_reg), 'bilateralreg_base.png');
imwrite(uint8(255*output_std_diffusion), 'bilateralstddiff_base.png');

imwrite(uint8(255*(.5 + source - output_std)), 'bilateralstd_detail.png');
imwrite(uint8(255*(.5 + source - output_reg)), 'bilateralreg_detail.png');
imwrite(uint8(255*(.5 + source - output_std_diffusion)), 'bilateralstddiff_detail.png');


% Apply lighting
LUTfile_id      = fopen('input_2','r');
doplut          = NaN(32768,1);
for n=1:32768
    doplut(n)   = str2double(fgetl(LUTfile_id));
end
fclose(LUTfile_id);

src_col = double(imread(src_col_name));
lighting = zeros(size(src_col));
ucontrast = zeros(size(src_col));

% /// bilateralstd
output = output_std;
gain_det = max(0,(source + .6*(source - output))./source); % max avoids NaN
gain_uco = max(0,(source +3.6*(source - output))./source); % max avoids NaN
gain_lum = max(0,(doplut( max(1,min(32768,round(output*(2^15-1) +1))) )/(2^15 -1))./output );

lighting(:,:,1) = src_col(:,:,1).*gain_det.*gain_lum;
lighting(:,:,2) = src_col(:,:,2).*gain_det.*gain_lum;
lighting(:,:,3) = src_col(:,:,3).*gain_det.*gain_lum;

imwrite(uint8(lighting),'bilateralstd_lighting.png');

ucontrast(:,:,1) = src_col(:,:,1).*gain_uco.*gain_lum;
ucontrast(:,:,2) = src_col(:,:,2).*gain_uco.*gain_lum;
ucontrast(:,:,3) = src_col(:,:,3).*gain_uco.*gain_lum;

imwrite(uint8(ucontrast),'bilateralstd_lighting_ucontrast.png');

% /// bilateralreg
output = output_reg;
gain_det = max(0,(source + .6*(source - output))./source); % max avoids NaN
gain_uco = max(0,(source +3.6*(source - output))./source); % max avoids NaN
gain_lum = max(0,(doplut( max(1,min(32768,round(output*(2^15-1) +1))) )/(2^15 -1))./output );

lighting(:,:,1) = src_col(:,:,1).*gain_det.*gain_lum;
lighting(:,:,2) = src_col(:,:,2).*gain_det.*gain_lum;
lighting(:,:,3) = src_col(:,:,3).*gain_det.*gain_lum;

imwrite(uint8(lighting),'bilateralreg_lighting.png');

ucontrast(:,:,1) = src_col(:,:,1).*gain_uco.*gain_lum;
ucontrast(:,:,2) = src_col(:,:,2).*gain_uco.*gain_lum;
ucontrast(:,:,3) = src_col(:,:,3).*gain_uco.*gain_lum;

imwrite(uint8(ucontrast),'bilateralreg_lighting_ucontrast.png');

% /// bilateralstddiff
output = output_std_diffusion;
gain_det = max(0,(source + .6*(source - output))./source); % max avoids NaN
gain_uco = max(0,(source +3.6*(source - output))./source); % max avoids NaN
gain_lum = max(0,(doplut( max(1,min(32768,round(output*(2^15-1) +1))) )/(2^15 -1))./output );

lighting(:,:,1) = src_col(:,:,1).*gain_det.*gain_lum;
lighting(:,:,2) = src_col(:,:,2).*gain_det.*gain_lum;
lighting(:,:,3) = src_col(:,:,3).*gain_det.*gain_lum;

imwrite(uint8(lighting),'bilateralstddiff_lighting.png');

ucontrast(:,:,1) = src_col(:,:,1).*gain_uco.*gain_lum;
ucontrast(:,:,2) = src_col(:,:,2).*gain_uco.*gain_lum;
ucontrast(:,:,3) = src_col(:,:,3).*gain_uco.*gain_lum;

imwrite(uint8(ucontrast),'bilateralstddiff_lighting_ucontrast.png');

end
