#include <cstdio>
#include <cstdlib>
#include <cstring>
#include "SparseFilter.h"
#include "IioImage.h"

using sparse_filter::IioImage;
using sparse_filter::SparseFilter;

// c: pointer to original argc
// v: pointer to original argv
// o: option name after hyphen
// d: default value (if NULL, the option takes no argument)
const char *pick_option(int *c, char **v, const char *o, const char *d) {
    int id = d ? 1 : 0;
    for (int i = 0; i < *c - id; i++) {
        if (v[i][0] == '-' && 0 == strcmp(v[i] + 1, o)) {
            char *r = v[i + id] + 1 - id;
            for (int j = i; j < *c - id; j++)
                v[j] = v[j + id + 1];
            *c -= id + 1;
            return r;
        }
    }
    return d;
}

int main(int argc, char *argv[]) {
    if (argc < 5) {
        fprintf(stderr, "too few parameters\n");
        fprintf(stderr, "usage: %s input sigma_d sigma_r output [-r r] [-t threshold] [-f filter_number (0:BF; 1:BFR; 2:SBF; 3:SBFR; 4:SBFR_lum)]\n", argv[0]);
        return 1;
    }
    IioImage source(argv[1]);
    IioImage output;
    float sigma_d = atof(argv[2]);
    float sigma_r = atof(argv[3]);
    int r = atoi(pick_option(&argc, argv, "r", "-1"));
    if (r == -1) r = int(3 * sigma_d);
    float threshold = atof(pick_option(&argc, argv, "t", "1"));

    int reg = atoi(pick_option(&argc, argv, "f", "0"));
	IioImage computedpx; // An image too see which pixels are effectively computed
	IioImage difference;

    // Trois fonctions sont possibles :
    // 1) Sparse-Filter (fonction de Nicola)
    // 2) BilateralFilter : filtre bilatéral standard
    // 3) BilateralFilterReg : filter bilatéral avec regression

	switch( reg ){
		case 0 :
			printf("Application of the Standard Bilateral Filter.\n");
			BilateralFilter(   source, sigma_d * sigma_d, sigma_r * sigma_r, r, &output);
			break;
			
		case 1 :
			printf("Application of the Bilateral Filter with Regression.\n");
			BilateralFilterReg(source, sigma_d * sigma_d, sigma_r * sigma_r, r, &output);
			break;
			
		case 2 :
			printf("Application of the Sparse Bilateral Filter.\n");
			printf("(Nicola Original Sparse Filter)\n");
			SparseFilter(      source, sigma_d * sigma_d, sigma_r * sigma_r, r, threshold, &output);
			break;
			
		case 3 :
			printf("Application of the Sparse Bilateral Filter with Regression\n");
			SBFR( source, sigma_d * sigma_d, sigma_r * sigma_r, r, threshold, &output, &computedpx);
			// printf("(Nothing here for now)\n");
			// return 0;
			break;
			
		case 4 :
			printf("Application of the Sparce Bilateral Filter with Regression\n");
			printf("Only the luminance is filtered, but bilateral weights are constructed according to the color image\n");
			SBFR2(source, sigma_d * sigma_d, sigma_r * sigma_r, r, threshold, &output, &computedpx);
			break;
			
		default :
			fprintf(stderr, "Incorrect filter_number.\n");
	}

	// Compute the difference between the filtered and the input image
	ComputeDiff(source, output, &difference);


    // Sauvegarde de l'image de sortie
    output.Save(argv[4]);
	computedpx.Save("debug_computed_pixels.png");
	difference.Save("debug_difference.png");
    return 0;
}
