% Thursday 28, January 2016 [CMLA]
% Charles H

function [img_out] = UBF_FFT(img_in,sigS,sigR,N)
% UBF_FFT: unnormalized fast bilateral filter, an FFT implementation.
% 
% Charles H. - 2016/01/28


% Measurements:
minI = min(img_in(:));
maxI = max(img_in(:));
% Segments
Lg_Seg = (maxI - minI)./(N-1);
Vl_Seg = minI + (0:N-1)*Lg_Seg;

% 9 Kernel in Fourier Domain
[H,W]       = size(img_in);
[fcol,frnk] = meshgrid( [0:(W-1),(-W):(-1)]/(2*W),...
                        [0:(H-1),(-H):(-1)]/(2*H) );
beta    = 2*(pi^2)*(sigS^2);
ker_w   = exp(-beta.*(fcol.^2 + frnk.^2));
ker_w   = ker_w/ker_w(1,1); % normalization

img_wF  = zeros(H,W);

for k=1:N
    
    img_d    = (img_in - Vl_Seg(k));                % d(x): difference
    img_p    = (1 - min(Lg_Seg,abs(img_d))/Lg_Seg); % for interpolation

    img_r    = exp(-(img_d.^2)/(2*sigR^2));         % for normalization
    img_f    = img_d.*img_r;                        % a slice
    
    % Mirror Copy
    img_f_mirror  = cat(1,img_f       ,img_f(H:-1:1,1:+1:W));
    img_f_mirror  = cat(2,img_f_mirror,img_f_mirror(1:+1:end,W:-1:1));
    
    % Fourier Transform
    img_f_FT = fft2(img_f_mirror);
    
    % Filtering and interpolation between slices
    % /// could (should) be done by DCT ///
    tmp = real(ifft2(img_f_FT.*ker_w));
    img_wF = img_wF + img_p.*tmp(1:H,1:W);
end

img_out = img_in + img_wF;
end
